import { __spread, __decorate, __metadata, __extends } from 'tslib';
import { Injectable, Component, NgModule, ErrorHandler } from '@angular/core';
import { MessageService } from 'primeng/components/common/messageservice';
import { RouterModule } from '@angular/router';
import { hasIn, isFunction, isEmpty, has, isArray } from 'lodash';
import { HttpErrorResponse } from '@angular/common/http';

var ErrorHandlerService = /** @class */ (function () {
    function ErrorHandlerService(messageService) {
        this.messageService = messageService;
        this._console = console;
    }
    ErrorHandlerService.prototype.handleError = function (error) {
        var errorLogger = this.getErrorLogger(error);
        var original = this.getOriginalError(error);
        var context = this.getDebugContext(error);
        var message = (original || error).message || "" + error;
        errorLogger(this._console, 'ERROR', message);
        if (original) {
            errorLogger(this._console, 'ORIGINAL ERROR', original);
        }
        if (context) {
            errorLogger(this._console, 'CONTEXT', context);
        }
        this.messageService.add({
            severity: 'error',
            summary: (original || error).constructor.name,
            detail: message,
            life: 15,
            closable: true,
            data: error,
        });
    };
    ErrorHandlerService.prototype.getDebugContext = function (error) {
        return error
            ? error.ngDebugContext ||
                this.getDebugContext(this.getOriginalError(error))
            : null;
    };
    ErrorHandlerService.prototype.getOriginalError = function (error) {
        var original = error;
        while (original) {
            original = original.ngOriginalError;
        }
        return original;
    };
    ErrorHandlerService.prototype.getErrorLogger = function (error) {
        return error.ngErrorLogger || this.defaultErrorLogger;
    };
    ErrorHandlerService.prototype.defaultErrorLogger = function (console) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        console.error.apply(console, __spread(values));
    };
    ErrorHandlerService.ctorParameters = function () { return [
        { type: MessageService }
    ]; };
    ErrorHandlerService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [MessageService])
    ], ErrorHandlerService);
    return ErrorHandlerService;
}());

var GenericErrorComponent = /** @class */ (function () {
    function GenericErrorComponent() {
    }
    GenericErrorComponent = __decorate([
        Component({
            selector: 'lib-generic-error',
            template: "<div class=\"card ui-message ui-messages-error\">\n  <h1>\n    <i class=\"ui-messages-icon fa fa-exclamation-triangle\"></i>\n    <span i18n class=\"ui-messages-summary\">Erreur !</span>\n  </h1>\n  <p i18n class=\"ui-messages-detail\">Une erreur s'est produite !</p>\n</div>\n"
        })
    ], GenericErrorComponent);
    return GenericErrorComponent;
}());

var NotFoundErrorComponent = /** @class */ (function () {
    function NotFoundErrorComponent() {
    }
    NotFoundErrorComponent = __decorate([
        Component({
            selector: 'lib-not-found-error',
            template: "<div class=\"card ui-message ui-messages-error\">\n  <h1>\n    <i class=\"ui-messages-icon fa fa-exclamation-triangle\"></i>\n    <span i18n class=\"ui-messages-summary\">Page non trouv\u00E9e !</span>\n  </h1>\n  <p i18n class=\"ui-messages-detail\">La ressource n'a pas \u00E9t\u00E9 trouv\u00E9e.</p>\n</div>\n"
        })
    ], NotFoundErrorComponent);
    return NotFoundErrorComponent;
}());

var routes = [
    {
        path: 'error/404',
        pathMatch: 'full',
        component: NotFoundErrorComponent,
    },
    { path: 'error/**', component: GenericErrorComponent },
];
var ErrorModule = /** @class */ (function () {
    function ErrorModule() {
    }
    ErrorModule = __decorate([
        NgModule({
            imports: [RouterModule.forChild(routes)],
            exports: [RouterModule],
            providers: [{ provide: ErrorHandler, useClass: ErrorHandlerService }],
            declarations: [GenericErrorComponent, NotFoundErrorComponent],
        })
    ], ErrorModule);
    return ErrorModule;
}());

function isErrorMessageHandler(that) {
    return hasIn(that, 'setError') && isFunction(that.setError);
}
function isConstraintViolationHandler(that) {
    return (hasIn(that, 'setConstraintViolations') &&
        isFunction(that.setConstraintViolations));
}
function isValidationErrorHandler(that) {
    return hasIn(that, 'setErrors') && isFunction(that.setErrors);
}
function isErrorLogger(that) {
    return hasIn(that, 'logError') && isFunction(that.logError);
}
function isErrorHandler(that) {
    return (isErrorMessageHandler(that) ||
        isConstraintViolationHandler(that) ||
        isValidationErrorHandler(that) ||
        isErrorLogger(that));
}
function isErrorClearer(that) {
    return hasIn(that, 'clearErrors') && isFunction(that.clearErrors);
}
function isDispatchableError(that) {
    return hasIn(that, 'dispatch') && isFunction(that.dispatch);
}

/**
 * Erreur "enveloppant" une autre erreur.
 */
var AppError = /** @class */ (function () {
    function AppError(original, message, stack, transient) {
        if (transient === void 0) { transient = false; }
        this.original = original;
        this.message = message;
        this.stack = stack;
        this.transient = transient;
        this.isNullError = false;
    }
    Object.defineProperty(AppError.prototype, "name", {
        get: function () {
            return this.getName();
        },
        enumerable: true,
        configurable: true
    });
    AppError.prototype.dispatch = function (handler) {
        return ((isErrorLogger(handler) &&
            handler.logError(this.message, this.name, this.stack)) ||
            (isErrorMessageHandler(handler) && handler.setError(this.message)) ||
            (isValidationErrorHandler(handler) &&
                handler.setErrors({ message: this.message })));
    };
    AppError.prototype.getName = function () {
        return ((this.original instanceof Error && this.original.name) ||
            this.constructor.name);
    };
    return AppError;
}());

/**
 * Supprime les erreurs d'un handler.
 */
function clearErrors(handler) {
    if (isErrorClearer(handler) && handler.clearErrors()) {
        return true;
    }
    var handled = false;
    if (isErrorMessageHandler(handler) && handler.setError(null)) {
        handled = true;
    }
    if (isValidationErrorHandler(handler) && handler.setErrors(null)) {
        handled = true;
    }
    if (isConstraintViolationHandler(handler) &&
        handler.setConstraintViolations(null)) {
        handled = true;
    }
    return handled;
}

/**
 * Erreur basée sur une réponse HTTP d'erreur.
 */
var HTTPError = /** @class */ (function (_super) {
    __extends(HTTPError, _super);
    function HTTPError(original, message) {
        return _super.call(this, original, message || original.message || original.statusText, undefined, original.status >= 500) || this;
    }
    Object.defineProperty(HTTPError.prototype, "status", {
        get: function () {
            return this.original.status;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HTTPError.prototype, "statusText", {
        get: function () {
            return this.original.statusText;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HTTPError.prototype, "headers", {
        get: function () {
            return this.original.headers;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HTTPError.prototype, "url", {
        get: function () {
            return this.original.url;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HTTPError.prototype, "error", {
        get: function () {
            return this.original.error;
        },
        enumerable: true,
        configurable: true
    });
    return HTTPError;
}(AppError));
/**
 * Erreur 403: Forbidden
 */
var AccessDeniedError = /** @class */ (function (_super) {
    __extends(AccessDeniedError, _super);
    function AccessDeniedError() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return AccessDeniedError;
}(HTTPError));
/**
 * Erreur 401: Authentication Required
 */
var AuthenticationError = /** @class */ (function (_super) {
    __extends(AuthenticationError, _super);
    function AuthenticationError() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return AuthenticationError;
}(HTTPError));
/**
 * Erreur 400: Bad Request
 */
var BadRequestError = /** @class */ (function (_super) {
    __extends(BadRequestError, _super);
    function BadRequestError() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return BadRequestError;
}(HTTPError));

/**
 * Erreur de validation retournée par api-platform.
 */
var ConstraintViolations = /** @class */ (function (_super) {
    __extends(ConstraintViolations, _super);
    function ConstraintViolations(original, message, violations) {
        var _this = _super.call(this, original, message) || this;
        _this.violations = violations;
        return _this;
    }
    ConstraintViolations.prototype.dispatch = function (handler) {
        return ((isConstraintViolationHandler(handler) &&
            handler.setConstraintViolations(this.violations)) ||
            _super.prototype.dispatch.call(this, handler));
    };
    ConstraintViolations.create = function (violations) {
        return new ConstraintViolations(new HttpErrorResponse({
            status: 400,
            statusText: 'Constraint violations',
        }), 'Constraint violations', violations);
    };
    return ConstraintViolations;
}(BadRequestError));

var ɵ0 = function (handler) {
    return clearErrors(handler);
};
/**
 * Singleton d'erreur null.
 */
var NullError = {
    isNullError: true,
    name: 'NullError',
    message: 'No error',
    dispatch: ɵ0,
};

/**
 * Transforme une valeur en erreur.
 *
 * Les informations intéressantes d'HttpErrorResponse et Error sont conservés.
 */
function normalizeError(err) {
    if (err instanceof AppError || err === NullError) {
        return err;
    }
    if (err instanceof HttpErrorResponse) {
        return normalizeHttpErrorResponse(err);
    }
    if (err instanceof Error) {
        return new AppError(err, err.message, err.stack, false);
    }
    if (isEmpty(err)) {
        return NullError;
    }
    if (typeof err === 'object') {
        return new AppError(err, extractMessage(err), err.stack, err.transient || false);
    }
    return new AppError(err, extractMessage(err));
}
function normalizeHttpErrorResponse(err) {
    var body = 'error' in err ? err.error : {};
    var message = extractMessage(body) || extractMessage(err);
    switch (err.status) {
        case 400:
            if (has(body, 'violations') && isArray(body.violations)) {
                return new ConstraintViolations(err, message, body.violations);
            }
            return new BadRequestError(err, message);
        case 401:
            return new AuthenticationError(err, message);
        case 403:
            return new AccessDeniedError(err, message);
        default:
            return new HTTPError(err, message);
    }
}
function extractMessage(err) {
    if (!err) {
        return undefined;
    }
    switch (typeof err) {
        case 'object':
            if ('message' in err) {
                return err.message;
            }
            if ('hydra:description' in err) {
                return err['hydra:description'];
            }
            if ('hydra:title' in err) {
                return err['hydra:title'];
            }
            if ('error' in err) {
                return extractMessage(err.error);
            }
            break;
        case 'string':
        case 'number':
            return "" + err;
    }
    return undefined;
}

/**
 * Erreur émise lorsqu'une exception ne peut pas être dispatchée.
 */
var UnhandledError = /** @class */ (function (_super) {
    __extends(UnhandledError, _super);
    function UnhandledError(original) {
        return _super.call(this, original, "Unhandled error: " + original.message, original.stack, false) || this;
    }
    return UnhandledError;
}(AppError));

var ValidationError = /** @class */ (function (_super) {
    __extends(ValidationError, _super);
    function ValidationError(errors) {
        var _this = _super.call(this, errors, errors.message || 'Validation error') || this;
        _this.errors = errors;
        return _this;
    }
    ValidationError.prototype.dispatch = function (handler) {
        return ((isValidationErrorHandler(handler) && handler.setErrors(this.errors)) ||
            _super.prototype.dispatch.call(this, handler));
    };
    return ValidationError;
}(AppError));

/*
 * Public API Surface of error-types
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AccessDeniedError, AppError, AuthenticationError, BadRequestError, ConstraintViolations, ErrorHandlerService, ErrorModule, GenericErrorComponent, HTTPError, NotFoundErrorComponent, NullError, UnhandledError, ValidationError, clearErrors, isConstraintViolationHandler, isDispatchableError, isErrorClearer, isErrorHandler, isErrorLogger, isErrorMessageHandler, isValidationErrorHandler, normalizeError, ɵ0 };
//# sourceMappingURL=devatscience-ngx-errors.js.map
